Functions¶
This section documents the system-defined scalar functions — named functions that you invoke using function-call syntax, such as COALESCE(a, b). Each operates on the values of a single row and returns a single value.
For functions that aggregate values across multiple rows, see aggregate functions.